interface StorageOptions {
  name?: string;
  /**
   * Expiration time in seconds.
   */
  expireIn?: number;
}

interface ConfigureOptions {
  window?: any;
  apiUrl?: string;
  timeout?: number;
  /**
   * @deprecated Use {@link storage} instead.
   */
  storageNamespace?: string;
  cookieDomain?: string;
  avoidCookies?: boolean;
  pk?: string;
  verbose?: boolean;
  storage?: StorageOptions;
  autoInjectTokenOnSubmit?: boolean;
  autoInjectTokenOnRequest?: boolean;
  formTokenInputName?: string;
  /**
   * Throttles custom events by 500ms and send them in batches. Enabled by default.
   */
  throttling?: boolean;
  eFn?: (e: any) => void; // debugging error logger
  wUrl?: string; // collector worker url
  dwUrl?: string; // debugg/er worker url
  dCss?: boolean; // disable inline css
  eCss?: boolean; // set external csses
}

interface Thenable<T> {
  then<U = T>(
    onFulfilled?: (value: T) => U | Thenable<U>,
    onRejected?: (error: any) => U | Thenable<U>
  ): Thenable<U>;

  catch<U = T>(onRejected?: (error: any) => U | Thenable<U>): Thenable<U>;

  finally(onFinally?: () => void): Thenable<T>;

  resolve(value: T): void;
  reject(error: any): void;
}

interface FormEventOnSubmitOptions {
  injectToken?: boolean;
}

interface UserParams {
  id: string;
  email?: string;
  phone?: string;
  registered_at?: string;
  name?: string;
  traits?: Record<string, any>;
  signature?: string;
}

type PageParams = {
  url?: string;
  name?: string;
  referrer?: string;
} & ({ user: UserParams; userJwt?: string } | { user?: UserParams; userJwt: string });

type FormParams = {
  name: string;
  values?: Record<string, string>;
} & ({ user: UserParams; userJwt?: string } | { user?: UserParams; userJwt: string });

type CustomParams = {
  name: string;
  properties?: Record<string, string>;
  user: UserParams;
} & ({ user: UserParams; userJwt?: string } | { user?: UserParams; userJwt: string });

interface ConfigureResponse {
  createRequestToken(): Thenable<string>;
  page(params: PageParams): Thenable<boolean | null>;
  form(params: FormParams): Thenable<boolean | null>;
  custom(params: CustomParams): Thenable<boolean | null>;
  formEventOnSubmit(
    event: Event,
    user: UserParams | string,
    opts?: FormEventOnSubmitOptions,
    onDone?: (event: Event, result: boolean | null) => any
  ): void | boolean;
  configure(options?: ConfigureOptions): ConfigureResponse;
}

declare function createRequestToken(): Thenable<string>;
declare function page(params: PageParams): Thenable<boolean | null>;
declare function form(params: FormParams): Thenable<boolean | null>;
declare function custom(params: CustomParams): Thenable<boolean | null>;
declare function injectTokenOnSubmit(event: Event, onDone?: (event: Event) => any): any;
declare function formEventOnSubmit(
  event: Event,
  user: UserParams | string,
  opts?: FormEventOnSubmitOptions,
  onDone?: (event: Event, result: boolean | null) => any
): void | boolean;
declare function configure(options?: ConfigureOptions): ConfigureResponse;

export { type ConfigureOptions, type CustomParams, type FormParams, type PageParams, type Thenable, type UserParams, configure, createRequestToken, custom, form, formEventOnSubmit, injectTokenOnSubmit, page };
